@@ -1,7 +1,7 @@
 # -*- perl -*-
 
 #
-# $Id: Tree.pm,v 5.6 2008/10/31 23:44:57 eserte Exp $
+# $Id: Tree.pm,v 5.8 2008/11/05 22:31:48 eserte Exp $
 # Author: Slaven Rezic
 #
 # Copyright (C) 2001,2004,2007,2008 Slaven Rezic. All rights reserved.
@@ -53,8 +53,8 @@ in a tree.
 =cut
 
 use strict;
-use vars qw($VERSION @ISA @POD %EXTRAPODDIR $FindPods $ExtraFindPods);
-$VERSION = sprintf("%d.%02d", q$Revision: 5.6 $ =~ /(\d+)\.(\d+)/);
+use vars qw($VERSION @ISA @POD %EXTRAPODDIR $ExtraFindPods);
+$VERSION = sprintf("%d.%02d", q$Revision: 5.8 $ =~ /(\d+)\.(\d+)/);
 
 use base 'Tk::Tree';
 
@@ -275,7 +275,7 @@ sub Populate {
 
 =over 4
 
-=item I<$tree>-E<gt>B<Fill>(?I<-nocache =E<gt> 1>?, ?I<-forked =E<gt> 0|1>?)
+=item I<$tree>-E<gt>B<Fill>(?I<-nocache =E<gt> 1>?, ?I<-forked =E<gt> 0|1>?, ?I<-fillcb =E<gt> ...>?)
 
 Find Pod modules and fill the tree widget. If I<-nocache> is
 specified, then no cache will be used for loading.
@@ -287,6 +287,9 @@ If C<-forked> is specified, then searching for Pods is done in the
 background, if possible. Note that the default is currently
 unspecified.
 
+A callback may be specified with the C<-fillcb> option and will be
+called after the tree is filled.
+
 =cut
 
 sub Fill {
@@ -299,8 +302,8 @@ sub Fill {
     }
 
     $w->delete("all");
-
-    $FindPods = Tk::Pod::FindPods->new unless $FindPods;
+    delete $w->{Pods};
+    $w->{Filled} = 0;
 
     my $forked = delete $args{-forked};
     if (!defined $forked) {
@@ -342,7 +345,7 @@ sub Fill {
 			      local $/;
 			      my $serialized = <$rdr>;
 			      my $pods = Storable::thaw($serialized);
-			      $w->_FillDone($pods);
+			      $w->_FillDone($pods, $args{'-fillcb'});
 			      $w->fileevent($rdr, 'readable', '');
 			      $w->Subwidget('UpdateLabel')->placeForget;
 			      $w->{FillPid} = undef;
@@ -353,7 +356,7 @@ sub Fill {
 
     # non-forked
     my $pods = $w->_FillFind(%args);
-    $w->_FillDone($pods);
+    $w->_FillDone($pods, $args{'-fillcb'});
 }
 
 sub _FillFind {
@@ -361,6 +364,7 @@ sub _FillFind {
 
     my $usecache = ($w->cget('-usecache') && !$args{'-nocache'});
 
+    my $FindPods = Tk::Pod::FindPods->new;
     my $pods = $FindPods->pod_find(-categorized => 1,
 				   -usecache => $usecache,
 				  );
@@ -378,11 +382,15 @@ sub _FillFind {
 	}
     }
 
+    if ($w->cget('-usecache') && !$FindPods->has_cache) {
+	$FindPods->WriteCache;
+    }
+
     $pods;
 }
 
 sub _FillDone {
-    my($w, $pods) = @_;
+    my($w, $pods, $fillcb) = @_;
 
     my %category_seen;
 
@@ -432,11 +440,12 @@ sub _FillDone {
 	}
     }
 
-    if ($w->cget('-usecache') && !$FindPods->has_cache) {
-	$FindPods->WriteCache;
-    }
-
+    $w->{Pods} = $pods;
     $w->{Filled}++;
+
+    if ($fillcb) {
+	$fillcb->();
+    }
 }
 
 sub folderentry {
@@ -493,8 +502,7 @@ sub SeePath {
     }
     DEBUG and warn "Call SeePath with $path\n";
     return if !$w->Filled; # not yet filled
-    return if !$FindPods;
-    my $pods = $FindPods->pods;
+    my $pods = $w->{Pods};
     return if !$pods;
 
     my $see_treepath = sub {
